/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of Mail Blaster.
 * 
 * Mail Blaster is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * Mail Blaster is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Mail Blaster. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.mailblaster.cli;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.Observable;
import java.util.Observer;

import com.gt.mailblaster.config.ConfigException;
import com.gt.mailblaster.config.SMTPConfig;
import com.gt.mailblaster.mailer.MailerException;
import com.gt.mailblaster.mailer.SMTPMailer;

/**
 * The command line interface for mail blaster. This is also the entry point (i.e. the class 
 * that has the main method and from where the application's thread of execution starts).
 * 
 * @author The Shoe Shine Boy
 *
 */
public class CLIMailBlaster implements Observer {

	/**
	 * The method that creates the SMTPMailer object and sends out mails based on 
	 * SMTPConfig object configuration. 
	 * 
	 * @param configFilePath	The Configuration File Path
	 * @throws MailerException	In case there is any exception during sending mail(s)
	 * @throws ConfigException	In case there is any exception loading the configuration
	 */
	public void sendMail(String configFilePath) throws MailerException, ConfigException {
		SMTPConfig config = new SMTPConfig();
		config.loadFrom(new File(configFilePath));
		
		SMTPMailer mailer = new SMTPMailer(config.getMailHost(), config.isAuthenticate(), 
				config.isEnableTLS(), config.getAcName(), config.getAcPwd());
		mailer.addObserver(this);
		
		mailer.mail(config.getTos(), config.getSubject(), config.getBody(), 
				config.isSendAsHTML(), config.getAttachments());
	}
	
	/**
	 * The application main method.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			if (args.length == 0) {
				System.out.println("Launching Config Builder...");
				ConfigBuilderFrame cbf = new ConfigBuilderFrame();
				String configFilePath = cbf.getConfigFilePath();
				if (configFilePath != null) {
					System.out.print("Do you want to send mail(s) using just created config file (y/n)? ");
					String input = new BufferedReader(new InputStreamReader(System.in)).readLine();
					input = input.toLowerCase().trim();
					if (input.equals("y")) {
						new CLIMailBlaster().sendMail(configFilePath);
					} else if (input.equals("n")) {
						// do nothing.
					} else {
						System.out.println("Invalid choice. Only 'y' or 'n' are allowed.");
					}
				}
			} else if (args.length == 1) {
				if (args[0].equals("-help")) {
					System.out.println("Usage: java -jar mailblaster.jar [-help] [config file path]");
				} else {
					new CLIMailBlaster().sendMail(args[0]);
				}
			} else {
				System.err.println("Usage: java -jar mailblaster.jar [-help] [config file path]");
			}
			System.out.println("Application Closed.");
			System.exit(0);
		} catch (Exception e) {
			System.err.println(e.getMessage());
			System.out.println("Application Closed.");
			System.exit(1);
		}
	}

	public void update(Observable arg0, Object arg1) {
		if (arg1 instanceof String) {
			System.out.println(arg1);
		} else {
			// skip this event.
		}
	}
}
