/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of Mail Blaster.
 * 
 * Mail Blaster is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * Mail Blaster is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Mail Blaster. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.mailblaster.mailer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.NoSuchProviderException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import com.sun.mail.smtp.SMTPTransport;

/**
 * The class that extends Abstract Mailer and provides a SMTP based mailing facility.
 * 
 * @author The Shoe Shine Boy
 *
 */
public class SMTPMailer extends AbstractMailer {
	// mail server and account related parameters
	private String m_mailHost, m_acName, m_acPwd;
	private boolean m_authenticate, m_enableTLS;
	
	/**
	 * Constructor
	 * 
	 * @param mailHost		The mail server hostname or IP address.
	 * @param authenticate	The boolean flag telling if authentication is required or not.
	 * @param enableTLS		The boolean flag telling if TLS is enabled or not.
	 * @param acName		The mail account to be used to blast mails.
	 * @param acPwd			Password of the mail account.
	 */
	public SMTPMailer(String mailHost, boolean authenticate, boolean enableTLS, 
			String acName, String acPwd) {
		m_mailHost = mailHost;
		m_authenticate = authenticate;
		m_enableTLS = enableTLS;
		m_acName = acName;
		m_acPwd = acPwd;
	}

	public void mail(String[] to, String subject, String body, boolean sendAsHTML, 
			File[] attachments) throws MailerException {
		// Connect to SMTP Server
		notifyObservers("Connecting to SMTP Server...");
		Properties props = new Properties();
		if (m_mailHost == null || m_mailHost.trim().length() == 0) {
			throw new MailerException("mail.host is null or empty.");
		}
		props.setProperty("mail.host", m_mailHost);
		props.setProperty("mail.smtp.auth", String.valueOf(m_authenticate));
		props.setProperty("mail.smtp.starttls.enable", String.valueOf(m_enableTLS));
		if (m_acName == null || m_acName.trim().length() == 0) {
			throw new MailerException("Mail Account Name is null or empty.");
		} else {
			// move on.
		}
		if (m_acPwd == null || m_acPwd.trim().length() == 0) {
			throw new MailerException("Mail Account Password is null or empty.");
		} else {
			// move on
		}
		Session mailSession = Session.getInstance(props, new Authenticator() {
			protected PasswordAuthentication getPasswordAuthentication() {
				return new PasswordAuthentication(m_acName, m_acPwd);
			}
		});
		
		// Create Message
		Transport transport;
		MimeMessage message;
		Multipart multiPart;
		try {
			transport = (SMTPTransport)mailSession.getTransport("smtp");
			transport.connect(m_mailHost, m_acName, m_acPwd);
			notifyObservers("Connected to " + m_mailHost);

			message = new MimeMessage(mailSession);
			if (subject == null || subject.trim().length() == 0) {
				throw new MailerException("Mail Subject is null or empty.");
			} else {
				message.setSubject(subject);
			}
			message.setFrom(new InternetAddress(m_acName));
			multiPart = new MimeMultipart();
			MimeBodyPart bodyPart = new MimeBodyPart();
			if (body == null || body.trim().length() == 0) {
				throw new MailerException("Mail body is null or empty.");
			} else {
				// move on.
			}
			if (sendAsHTML) {
				bodyPart.setContent(body, "text/html");
			} else {
				bodyPart.setText(body);
			}
			multiPart.addBodyPart(bodyPart);
		} catch (AddressException e) {
			throw new MailerException(e);
		} catch (NoSuchProviderException e) {
			throw new MailerException(e);
		} catch (MessagingException e) {
			throw new MailerException(e);
		}
	    if (attachments == null) {
	    	// do nothing
	    } else {
	        for (int j=0; j<attachments.length; j++) {
	    		try {
	    			if (attachments[j].exists()) {
						String attachmentName = attachments[j].getName();
						String attachmentPath = attachments[j].getCanonicalPath();
						MimeBodyPart attachmentPart = new MimeBodyPart();
						attachmentPart.setDataHandler(new DataHandler(new FileDataSource(attachmentPath)));
						attachmentPart.setFileName(attachmentName);
						multiPart.addBodyPart(attachmentPart);
	    			} else {
	    				throw new FileNotFoundException("cannot find file " + attachments[j].getCanonicalPath());
	    			}
				} catch (IOException e) {
					notifyObservers("Skipping attachment '" + attachments[j].getName() + 
							"' because of error: " + e.getMessage());
				} catch (MessagingException e) {
					notifyObservers("Skipping attachment '" + attachments[j].getName() + 
							"' because of error: " + e.getMessage());
				}
	        }
	    }
        try {
			message.setContent(multiPart);
			message.addRecipient(Message.RecipientType.TO, new InternetAddress(m_acName));
			if (to == null || to.length == 0) {
				throw new MailerException("List of recipients is null or empty.");
			} else {
				// move on.
			}
			for (int i=0; i<to.length; i++) {
				message.addRecipient(Message.RecipientType.BCC, new InternetAddress(to[i]));
			}

			// Send email
			notifyObservers("Sending mail(s)...");
			Transport.send(message);
			notifyObservers("Mail(s) sent successfully.");
			
			// Close connection to the SMTP Server
			notifyObservers("Closing Connection to SMTP Server...");
			transport.close();
			notifyObservers("Mail(s) sent. Connection to SMTP Server closed.");
		} catch (AddressException e) {
			throw new MailerException(e);
		} catch (MessagingException e) {
			throw new MailerException(e);
		}
	}

}
